#!/bin/sh
# (c) 2004 by Jonas Meurer <jonas@freesources.org> | licensed under the GNU/GPL
# this script automaticly regenerates lurker.conf with the information from list_lists(8) and
# the original lurker.conf. to use it correctly, you should have set the two lurker-mailman
# compability vars in /etc/lurker/lurker.conf.mm, se Automatic Configuation Section at
# REAMDE.Debian for more information.

# hyphens within listnames are transformed to underscores internally, but they are converted back
# before being written to lurkers configuration file. thus, users should't realize that fact.

set +x 

# configuration
tmpdir=/tmp
lurker_conf=/etc/lurker/lurker.conf
lurker_conf_tmp=${tmpdir}/lurker.conf
lists_tmp=${tmpdir}/listed-lists
additional_lists=${tmpdir}/other-lists
mm_command="/usr/lib/mailman/bin/list_lists -a"

listhost=`grep "^#[[:space:]]*listhost" ${lurker_conf}.mm | awk -F "=[[:space:]]*" {'print $2'}`
listurl=`grep "^#[[:space:]]*listurl" ${lurker_conf}.mm | awk -F "=[[:space:]]*" {'print $2'}`
[ -e $listhost ] && listhost="lists.not-configured.yet"
[ -e $listurl ] && listurl="http://lists.not-configured.yet/path/to"

if [ -f ${tmpdir}/lurker.conf ]; then
   mv ${lurker_conf_tmp} ${lurker_conf_tmp}.old
fi

initialize_lurker_conf() {
	sed -e '/^[[:space:]]*group[[:space:]]*=/,$d' $lurker_conf > $lurker_conf_tmp
	echo "" >> $lurker_conf_tmp
}

read_lurker_conf() {
	while read line; do
	  #echo "LINE: $line"
	  if echo $line | grep -q "^[[:space:]]*group[[:space:]]*="; then
	    set -- $line; group=$3; groups="$groups $group"
	  elif echo $line | grep -q "^[[:space:]]*heading[[:space:]]*="; then
	    set -- $line; name="$1"; shift 2; value="$@"; eval ${group}_${name}=\"$value\";
	    # echo -n " ${group}_heading = "; eval echo "\$${group}_heading"
	  elif echo $line | grep -q "^[[:space:]]*list[[:space:]]*="; then
	    set -- $line; list=${3//-/_}
	    eval ${group}=\"\$${group} $list\"
	    # echo -n " ${group} = "; eval echo -n "\$${group}"
	    # echo " | CURRENT LIST: ${list//_/-}"
	  elif echo $line | egrep -q "^[[:space:]]*(title|address|link|description)[[:space:]]*="; then
	    set -- $line; name="$1"; shift 2; value="$@"; eval ${group}_${list}_${name}=\"$value\";
	    # echo -n "  ${group}_${list}_${name} = "; eval echo "\$${group}_${list}_${name}"
	  else
	    case "$line" in
	       \#*|"")
	         continue
	       ;;
	       include*)
	         end="$end
$line"
	       ;;
	    esac
	  fi
	done < $lurker_conf
}

read_mailman() {
	eval $mm_command |grep -v "matching mailing lists found:" > $lists_tmp
	if [ -f $additional_lists ]; then
	  cat $additional_lists >> $lists_tmp
	fi
	while read line; do
		list=`echo "$line" |awk -F "[[:space:]]-[[:space:]]" '{print $1}'| tr [[:upper:]] [[:lower:]]`
		desc=`echo $line |awk -F "[[:space:]]-[[:space:]]" '{print $2}'`
		for group in $groups; do
		  list_test=`echo ${!group} | grep "${list//-/_}"`
		  if [ "$list_test" ]; then
		    flag="1";
		    echo "LIST ALREADY EXISTS IN LURKER.CONF: $list in $group"
		    if echo $desc | grep -vq "[no description available]"; then
		      echo "DESCRIPTION IS UPDATED TO: $desc"
		      eval ${group}_${list//-/_}_description="$desc"
		    fi
		  fi
		  list_test="" 
		done
		if [ "$flag" != "1" ]; then
		  echo "LIST DOESN'T EXIST IN LURKER.CONF UNTIL NOW: $list"
		  [ -e $tempgroup ] && tempgroup="tempgroup$((RANDOM%1000+1))" && groups="$groups $tempgroup"
		  echo "WE'LL ADD IT IN A TEMPORARLY CREATED GROUP: $list in $tempgroup"  
		  echo "GROUPS: $groups"
		  eval ${tempgroup}=\"\$${tempgroup} $list\"
		  eval ${tempgroup}_heading=\"$group\"
		  eval ${tempgroup}_${list//-/_}_title=\"$list\"
		  eval ${tempgroup}_${list//-/_}_link=\"$listurl/$list\"
		  eval ${tempgroup}_${list//-/_}_address=\"$list@$listhost\"
		  eval ${tempgroup}_${list//-/_}_description=\"$desc\"
		else
		  flag=0;
		fi
	done < $lists_tmp
}

write_lurker_conf() {
	initialize_lurker_conf;
	for group in $groups; do
	  echo "" >> $lurker_conf_tmp
	  echo "group = $group" >> $lurker_conf_tmp
	  x=${group}_heading
	  echo "  heading = ${!x}" >> $lurker_conf_tmp
	  for list in ${!group}; do
	    xt=${group}_${list}_title
	    xa=${group}_${list}_address
	    xl=${group}_${list}_link
	    xd=${group}_${list}_description
	    echo "" >> $lurker_conf_tmp
	    echo "  list = ${list//_/-}" >> $lurker_conf_tmp
	    echo "       title       = ${!xt}" >> $lurker_conf_tmp
	    echo "       address     = ${!xa}" >> $lurker_conf_tmp
	    echo "       link        = ${!xl}" >> $lurker_conf_tmp
	    echo "       description = ${!xd}" >> $lurker_conf_tmp
	  done
	done
	echo "" >> $lurker_conf_tmp
	echo $end >> $lurker_conf_tmp
}

case "$1" in
  create)
    read_mailman;
    write_lurker_conf;
  ;;
  update)
    read_lurker_conf;
    read_mailman;
    write_lurker_conf;
  ;;
esac

exit 0
